"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.apiDockerImageAwsCli = exports.runTask = exports.startTask = exports.getTaskStatus = exports.getLogs = void 0;
const ecs_1 = __importDefault(require("aws-sdk/clients/ecs"));
const cloudwatchlogs_1 = __importDefault(require("aws-sdk/clients/cloudwatchlogs"));
const assert_1 = __importDefault(require("assert"));
const infra_aws_1 = require("@goldstack/infra-aws");
const infra_1 = require("@goldstack/infra");
const createECS = async (deployment) => {
    const awsUser = await (0, infra_aws_1.getAWSUser)(deployment.awsUser);
    const credentials = await (0, infra_aws_1.getAWSCredentials)(awsUser);
    return new ecs_1.default({
        apiVersion: '2014-11-13',
        credentials,
    });
};
const createCloudWatchLogs = async (deployment) => {
    const awsUser = await (0, infra_aws_1.getAWSUser)(deployment.awsUser);
    const credentials = await (0, infra_aws_1.getAWSCredentials)(awsUser);
    return new cloudwatchlogs_1.default({
        credentials,
    });
};
const getLogs = async (params) => {
    const ecs = await createECS(params.deployment);
    const deploymentState = params.deploymentState;
    const taskId = params.taskId;
    const tasks = await ecs
        .describeTasks({
        tasks: [taskId],
        cluster: (0, infra_1.readTerraformStateVariable)(deploymentState, 'cluster_name'),
    })
        .promise();
    (0, assert_1.default)(tasks.tasks);
    const task = tasks.tasks[0];
    (0, assert_1.default)(task.taskDefinitionArn);
    const taskDefinitionResponse = await ecs
        .describeTaskDefinition({
        taskDefinition: task.taskDefinitionArn.split('/')[1],
    })
        .promise();
    const taskDefinition = taskDefinitionResponse.taskDefinition;
    (0, assert_1.default)(taskDefinition);
    const containerDefinitions = taskDefinition.containerDefinitions;
    (0, assert_1.default)(containerDefinitions);
    const logConfiguration = containerDefinitions[0].logConfiguration;
    (0, assert_1.default)(logConfiguration);
    (0, assert_1.default)(logConfiguration.options);
    const awsLogsGroup = logConfiguration.options['awslogs-group'];
    const awsLogsPrefix = logConfiguration.options['awslogs-stream-prefix'];
    const cwlogs = await createCloudWatchLogs(params.deployment);
    const logEvents = await cwlogs
        .getLogEvents({
        logGroupName: awsLogsGroup,
        logStreamName: `${awsLogsPrefix}/main/${taskId}`,
        limit: params.limit,
        nextToken: params.nextToken,
        startTime: params.startTime,
        endTime: params.endTime,
    })
        .promise();
    return logEvents.events || [];
};
exports.getLogs = getLogs;
const getTaskStatus = async (deployment, taskExecutionArn) => {
    const ecs = await createECS(deployment);
    const tasksState = await ecs
        .describeTasks({ tasks: [taskExecutionArn] })
        .promise();
    if (tasksState.failures && tasksState.failures.length > 0) {
        throw new Error(`Cannot determine state of launched tast. ${tasksState.failures[0].reason}`);
    }
    (0, assert_1.default)(tasksState.tasks);
    return tasksState.tasks[0];
};
exports.getTaskStatus = getTaskStatus;
const startTask = async (params) => {
    const deployment = params.deployment;
    const ecs = await createECS(params.deployment);
    const config = params.config;
    const deploymentState = params.deploymentState;
    if (!deploymentState) {
        throw new Error(`Deployment state for '${params.deployment.name}' is not defined. Make sure to have deployed the image to this environment.`);
    }
    const repo = (0, infra_1.readTerraformStateVariable)(deploymentState, 'repo_url');
    let imageName;
    let imageHash = params.imageHash;
    if (!params.imageHash) {
        if (!deploymentState['latest']) {
            throw new Error(`Cannot run image since image has not been deployed for deployment '${params.deployment.name}'.`);
        }
        imageName = deploymentState['latest'];
        imageHash = deploymentState['latest'].split(':')[1];
    }
    else {
        imageName = `${repo}:${imageHash}`;
    }
    const awsLogsGroup = '/ecs/' + repo + '/' + imageHash;
    const taskDefinition = (await ecs
        .registerTaskDefinition({
        containerDefinitions: [
            {
                name: 'main',
                command: params.cmd,
                environment: params.env,
                image: imageName,
                logConfiguration: {
                    logDriver: 'awslogs',
                    options: {
                        'awslogs-create-group': 'true',
                        'awslogs-region': deployment.awsRegion,
                        'awslogs-group': awsLogsGroup,
                        'awslogs-stream-prefix': 'ecs',
                    },
                },
            },
        ],
        taskRoleArn: (0, infra_1.readTerraformStateVariable)(deploymentState, 'ecs_task_role_arn'),
        networkMode: 'awsvpc',
        cpu: '1 vcpu',
        memory: '4 GB',
        executionRoleArn: (0, infra_1.readTerraformStateVariable)(deploymentState, 'ecs_task_execution_role_arn'),
        requiresCompatibilities: ['FARGATE'],
        family: `temp-${config.configuration.imageTag}-${new Date().getTime()}-${imageHash}`,
    })
        .promise()).taskDefinition;
    (0, assert_1.default)(taskDefinition);
    const clusterName = (0, infra_1.readTerraformStateVariable)(deploymentState, 'cluster_name');
    const runTaskResponse = await ecs
        .runTask({
        cluster: clusterName,
        count: 1,
        launchType: 'FARGATE',
        networkConfiguration: {
            awsvpcConfiguration: {
                subnets: [(0, infra_1.readTerraformStateVariable)(deploymentState, 'subnet')],
                assignPublicIp: 'ENABLED',
                securityGroups: [
                    (0, infra_1.readTerraformStateVariable)(deploymentState, 'container_security_group'),
                ],
            },
        },
        taskDefinition: `${taskDefinition.family}:${taskDefinition.revision}`,
        // command does not need to overriden here since it is defined in the task definition
        overrides: {},
    })
        .promise();
    const tasks = runTaskResponse.tasks;
    if (!tasks) {
        throw new Error(`Task definitions not supplied after running define tasks for ${imageName}`);
    }
    const taskExecution = tasks[0];
    (0, assert_1.default)(taskExecution.taskArn);
    const taskId = taskExecution.taskArn.split('/')[1];
    const cloudWatchLink = `https://${deployment.awsRegion}.console.aws.amazon.com/cloudwatch/home?region=${deployment.awsRegion}#logsV2:log-groups/log-group/${awsLogsGroup.replace(/\//g, '$252F')}/log-events/${`ecs/main/${taskExecution.taskArn.split('/')[2]}`
        .replace('$', '$2524')
        .replace('[', '$255B')
        .replace(']', '$255D')
        .replace(/\//g, '$252F')}`;
    const ecsConsoleLink = `https://${deployment.awsRegion}.console.aws.amazon.com/ecs/home?region=${deployment.awsRegion}#/clusters/${clusterName}/tasks/${taskExecution.taskArn.split('/')[2]}/details`;
    return {
        taskArn: taskExecution.taskArn,
        taskId,
        awsLogsGroup,
        ecsConsoleLink,
        awsLogsConsoleLink: cloudWatchLink,
    };
};
exports.startTask = startTask;
const runTask = async (params) => {
    var _a, _b;
    const deployment = params.deployment;
    const ecs = await createECS(deployment);
    const result = await (0, exports.startTask)(params);
    const taskId = result.taskArn.split('/')[1];
    console.log(`Started task ${result.taskArn}`);
    const deploymentState = params.deploymentState;
    let tasks = await ecs
        .describeTasks({
        tasks: [taskId],
        cluster: (0, infra_1.readTerraformStateVariable)(deploymentState, 'cluster_name'),
    })
        .promise();
    (0, assert_1.default)(tasks.tasks);
    let task = tasks.tasks[0];
    console.log(`Task status: ${task.lastStatus}`);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const waitForStateRunning = 'tasksRunning';
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const waitForParams = {
        tasks: [taskId],
        cluster: (0, infra_1.readTerraformStateVariable)(deploymentState, 'cluster_name'),
    };
    await ecs.waitFor(waitForStateRunning, waitForParams).promise();
    console.log('Task started up successfully ...');
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const waitForStateStopped = 'tasksStopped';
    await ecs.waitFor(waitForStateStopped, waitForParams).promise();
    console.log('Task stopped on ECS.');
    tasks = await ecs
        .describeTasks({
        tasks: [taskId],
        cluster: (0, infra_1.readTerraformStateVariable)(deploymentState, 'cluster_name'),
    })
        .promise();
    (0, assert_1.default)(tasks.tasks);
    task = tasks.tasks[0];
    const taskDefinition = {
        family: (_a = task.taskDefinitionArn) === null || _a === void 0 ? void 0 : _a.split('/')[1].split(':')[0],
        revision: (_b = task.taskDefinitionArn) === null || _b === void 0 ? void 0 : _b.split('/')[1].split(':')[1],
    };
    const logEvents = await (0, exports.getLogs)({
        deployment,
        taskId,
        deploymentState,
        config: params.config,
    });
    console.log('Logs from container:');
    for (const event of logEvents) {
        console.log(`> ${event.timestamp} ${event.message}`);
    }
    await ecs
        .deregisterTaskDefinition({
        taskDefinition: `${taskDefinition.family}:${taskDefinition.revision}`,
    })
        .promise();
    (0, assert_1.default)(task.containers);
    const taskContainer = task.containers[0];
    const exitCode = taskContainer.exitCode;
    console.log(`Container exited with code ${exitCode}`);
    if (exitCode !== 0) {
        throw new Error(`Error running container: ${taskContainer.reason}`);
    }
    return result;
};
exports.runTask = runTask;
const apiDockerImageAwsCli = async (config, deployment, args) => {
    if (args.length < 1) {
        throw new Error('Please specify an operation along with the image command. Supported operations ["run"]');
    }
    const [, deploymentName] = args;
    const deploymentState = (0, infra_1.readDeploymentState)('./', deploymentName);
    const operation = args[0];
    if (operation === 'run') {
        const [, , ...remainingArgs] = args;
        const taskArn = await (0, exports.runTask)({
            cmd: remainingArgs,
            deployment,
            deploymentState,
            config,
        });
        console.log(taskArn);
        return;
    }
    if (operation === 'start') {
        const [, , ...remainingArgs] = args;
        const taskArn = await (0, exports.startTask)({
            cmd: remainingArgs,
            deployment,
            deploymentState,
            config,
        });
        console.log(taskArn);
        return;
    }
    if (operation === 'logs') {
        const [, , taskArn] = args;
        const logEvents = await (0, exports.getLogs)({
            deployment,
            taskId: taskArn.split('/')[1],
            deploymentState,
            config,
        });
        for (const event of logEvents) {
            console.log(`${event.timestamp} ${event.message}`);
        }
        return;
    }
    throw new Error(`Unknown operation for image command: ${operation}`);
};
exports.apiDockerImageAwsCli = apiDockerImageAwsCli;
//# sourceMappingURL=apiDockerImageAws.js.map